# FITS MULTISPECIES N-MIXTURE MODEL


# load required libraries
library(mvtnorm)
library(coda)



# define utility functions
logit = function(xx) { log(xx) - log(1-xx) }
expit = function(xx) { 1/(1 + exp(-xx)) }

LowerTriangle = function(x, n) {
    # reconstruct lower-triangular matrix and diagonal from its nonzero elements stored in column-major order
    L = matrix(0, nrow=n, ncol=n)
    p = 0
    for (j in 1:n) {
        for (i in j:n) {
            p = p+1
            L[i,j] = x[p]
        }
    }
    L
}


mcse = function(x, FUNC,  batchsize=floor(sqrt(length(x))))  {

  # Compute Monte Carlo standard error of a functional (FUNC) of a Markov chain (x) using the Subsampling Bootstrap Method (SBM)
  n = length(x)
  nb = n - batchsize + 1
  bval = rep(NA,nb)
  for (j in 1:nb) {
    ind = j:(batchsize+j-1)
    bval[j] = FUNC(x[ind])
  }
  ##  var.bval = var(bval)*(nb-1) * n * batchsize / ( (n-batchsize) * nb )    #  OLBM method
  var.bval = var(bval)*(nb-1) * batchsize / nb
  
  list(se=sqrt(var.bval / n), batchsize=batchsize)
}

lowerQuantile = function(z) {  quantile(z, probs=.025)  }
upperQuantile = function(z) {  quantile(z, probs=.975)  }




# read detection data from files
d = read.csv('Askins-period-new-observer-1-27-14.csv')
d = d[!is.na(d[,'period']), ]

site = as.integer(d[,'point'])
date = as.character(d[,'date'])
observer = as.character(d[,'observer'])
weather = as.character(d[,'weather'])
timeOfDay = as.integer(d[,'time'])
timePeriod = as.character(d[,'time2'])


# extract data for unique sampling events
survey = unique(data.frame(site,date,observer,weather,timeOfDay,timePeriod))
dimnames(survey)[[1]] = 1:dim(survey)[1]
I = dim(survey)[1]
J = rep(4, I)

# compute year and day of year from date
dateVec = as.Date(survey[,'date'], format='%m/%d/%Y')
year = substr(dateVec, 1,4)
yrIndex = (year=='1983')
dayOfYear = rep(NA, I)
dayOfYear[yrIndex] = julian(dateVec[yrIndex], origin=as.Date('1983-01-10'))
dayOfYear[!yrIndex] = julian(dateVec[!yrIndex], origin=as.Date('1984-01-10'))
survey = cbind(survey, year, dayOfYear)

# compute species-specific counts for each sampling event
species.Number = sort(unique(d[,'species.num']))
K = length(species.Number)

y = array(0, dim=c(I,K,max(J)))

for (i in 1:I) {
    ind = d[,'point']==survey[i,'site'] & as.character(d[,'date'])==as.character(survey[i,'date']) & as.character(d[,'observer'])==as.character(survey[i,'observer']) &  as.character(d[,'weather'])==as.character(survey[i,'weather']) & as.integer(d[,'time'])==as.integer(survey[i,'timeOfDay']) & as.character(d[,'time2'])==as.character(survey[i,'timePeriod'])
    
    dsub = d[ind, c('species.num','individual')]

    for (k in 1:K) {
        ind.species = dsub[,'species.num']==species.Number[k]
        if (sum(ind.species)>0) {
            individual.species = dsub[ind.species,'individual']
            nindividuals = table(table(individual.species))
            nperiods = as.integer(names(nindividuals))
            y[i,k, nperiods] = nindividuals
        }
    }
}

# compute total number of individuals detected for each sampling event and species
ysum = matrix(nrow=I, ncol=K)
for (i in 1:I) {
    ysum[i, ] = t(apply(y[i,,], 1, sum, na.rm=TRUE))
}


# read in site-specific data (e.g., habitat covariate measurements);
# then merge with sampling-event data
d = read.csv('Askins-by-point-wcov-11-5-12.csv')
colInd = c(1,4:19)
hab = d[1, colInd]
for (i in 1:I) {
    rowInd = d[,'point']==survey[i,'site']
    if (i==1) {
        hab[1,] = d[rowInd, colInd]
    }
    else {
        hab = rbind(hab, d[rowInd, colInd])
    }
}
survey = cbind(survey, hab)




# construct covariate matrices for analysis

landVars = c('area..ha.', 'donut.area', 'isolat..km.', 'regional.forest.area')
land = survey[, landVars]
land.pca = prcomp(land, center=TRUE, scale=TRUE, retx=TRUE)
landPC1 = land.pca$x[,1]
landPC2 = land.pca$x[,2]

vegVars = c('ave.canopy.height', 'basal.area.dead.trees', 'basal.area.trees', 'number.herb.species', 'number.tree.species', 'percent..cover.trees', 'percent.cover.herbs', 'percent.cover.shrubs', 'relative.dominance.conifers', 'species.diversity.trees.Shannon', 'tree.density')
veg = survey[, vegVars]
veg.pca = prcomp(veg, center=TRUE, scale=TRUE, retx=TRUE)
vegPC1 = veg.pca$x[,1]
vegPC2 = veg.pca$x[,2]
vegPC3 = veg.pca$x[,3]
vegPC4 = veg.pca$x[,4]
vegPC5 = veg.pca$x[,5]
vegPC6 = veg.pca$x[,6]


# ... assign potential species-specific covariates of abundance to xmat
oneVec = matrix(1,nrow=I,ncol=1)
xmat = cbind(oneVec, landPC1, landPC2, vegPC1, vegPC2, vegPC3, vegPC4, vegPC5, vegPC6)
dimnames(xmat)[[2]][1] = 'Intercept'

# ... assign potential species-specific covariates of detection to wmat
doy = (survey[,'dayOfYear'] - mean(survey[,'dayOfYear'])) / sd(survey[,'dayOfYear'])
survey = cbind(survey, doy)
wmat = model.matrix(~timePeriod+doy, data=survey)


# ... for each of K species, assign columns of xmat to be included in the model
xmatCol = matrix(nrow=5, ncol=K) 
xmatCol[1:1, 1] = c(1)
xmatCol[1:2, 2] = c(1,4)
xmatCol[1:2, 3] = c(1,5)
xmatCol[1:3, 4] = c(1,2,3)
xmatCol[1:4, 5] = c(1,2,3,5)
xmatCol[1:1, 6] = c(1)
xmatCol[1:2, 7] = c(1,4)
xmatCol[1:1, 8] = c(1)
xmatCol[1:1, 9] = c(1)
xmatCol[1:1, 10] = c(1)
xmatCol[1:2, 11] = c(1,2)
xmatCol[1:1, 12] = c(1)
xmatCol[1:3, 13] = c(1,2,5)
xmatCol[1:1, 14] = c(1)
xmatCol[1:1, 15] = c(1)
xmatCol[1:1, 16] = c(1)
xmatCol[1:1, 17] = c(1)
xmatCol[1:1, 18] = c(1)
xmatCol[1:1, 19] = c(1)
xmatCol[1:2, 20] = c(1,2)
xmatCol[1:1, 21] = c(1)
xmatCol[1:2, 22] = c(1,2)
xmatCol[1:1, 23] = c(1)
xmatCol[1:1, 24] = c(1)
xmatCol[1:1, 25] = c(1)
xmatCol[1:1, 26] = c(1)
xmatCol[1:3, 27] = c(1,5,6)
xmatCol[1:2, 28] = c(1,2)
xmatCol[1:2, 29] = c(1,2)
xmatCol[1:2, 30] = c(1,2)
xmatCol[1:2, 31] = c(1,2)
xmatCol[1:1, 32] = c(1)
xmatCol[1:1, 33] = c(1)
xmatCol[1:2, 34] = c(1,2)
xmatCol[1:4, 35] = c(1,2,3,5)
xmatCol[1:3, 36] = c(1,3,5)
xmatCol[1:2, 37] = c(1,2)
xmatCol[1:2, 38] = c(1,2)
xmatCol[1:2, 39] = c(1,2)
xmatCol[1:2, 40] = c(1,2)
xmatCol[1:4, 41] = c(1,2,4,5)
xmatCol[1:2, 42] = c(1,4)
xmatCol[1:2, 43] = c(1,2)
xmatCol[1:3, 44] = c(1,2,5)
xmatCol[1:3, 45] = c(1,2,3)
xmatCol[1:2, 46] = c(1,4)
xmatCol[1:3, 47] = c(1,2,5)
xmatCol[1:1, 48] = c(1)
xmatCol[1:2, 49] = c(1,2)
xmatCol[1:4, 50] = c(1,2,3,5)
xmatCol[1:1, 51] = c(1)
xmatCol[1:5, 52] = c(1,2,4,5,7)
xmatCol[1:3, 53] = c(1,2,7)
xmatCol[1:1, 54] = c(1)
xmatCol[1:1, 55] = c(1)
xmatCol[1:3, 56] = c(1,2,3)
xmatCol[1:1, 57] = c(1)
xmatCol[1:2, 58] = c(1,2)
xmatCol[1:1, 59] = c(1)
xmatCol[1:2, 60] = c(1,2)
xmatCol[1:2, 61] = c(1,2)
xmatCol[1:1, 62] = c(1)
xmatCol[1:1, 63] = c(1)
xmatCol[1:1, 64] = c(1)
xmatCol[1:2, 65] = c(1,4)
xmatCol[1:1, 66] = c(1)
xmatCol[1:1, 67] = c(1)
xmatCol[1:3, 68] = c(1,2,3)
xmatCol[1:1, 69] = c(1)
xmatCol[1:1, 70] = c(1)
xmatCol[1:1, 71] = c(1)
xmatCol[1:1, 72] = c(1)
xmatCol[1:1, 73] = c(1)

nx = apply(!is.na(xmatCol), 2, sum)  # number of regressors of abundance for each species



# ... for each of K species, assign columns of wmat to be included in the model
wmatCol = matrix(nrow=4, ncol=K)
wmatCol[1, ] = c(1)
wmatCol[2:2, c(8,14,22,23,33,44,47,48,51,55,59)] = c(4)

nw = apply(!is.na(wmatCol), 2, sum)  # number of regressors of detection probability for each species




# ... assign species to be included in the model
species.ind = match(c(22,28,32,39,42,50,54,57), species.Number)
y = y[, species.ind, ]
ysum = ysum[, species.ind]
K = dim(ysum)[2]

xmatCol = xmatCol[, species.ind]
if (!is.matrix(xmatCol)) xmatCol = matrix(xmatCol, ncol=K)  # check if all species have only intercept
nx = nx[species.ind]
wmatCol = wmatCol[, species.ind]
if (!is.matrix(wmatCol)) wmatCol = matrix(wmatCol, ncol=K)  # check if all species have only intercept
nw = nw[species.ind]



# fit model

# ... compute parameter guesses using single-species N-mixture models

negLL.MultinomPoissonMixture = function(param, yArg, xArg, wArg, Jarg) {
    beta = param[1:dim(xArg)[2]]
    alpha = param[(1+dim(xArg)[2]):(dim(xArg)[2]+dim(wArg)[2])]
    
    pVec =  expit(wArg %*% alpha)
    lambdaVec = exp(xArg %*% beta)
    I = dim(yArg)[1]

    logL = rep(NA, I)
    for (i in 1:I) {
        pMult = dbinom(c(1:J[i]), size=J[i], prob=pVec[i])
        logL[i] = sum(dpois(yArg[i,1:J[i]], lambda=lambdaVec[i]*pMult, log=TRUE))
    }
    (-1)*sum(logL)
}


deviance.MultinomPoissonMixture = function(param, ysumArg, xArg, wArg, Jarg) {
    beta = param[1:dim(xArg)[2]]
    alpha = param[(1+dim(xArg)[2]):(dim(xArg)[2]+dim(wArg)[2])]
    
    pVec =  expit(wArg %*% alpha)
    lambdaVec = exp(xArg %*% beta)
    I = length(ysumArg)

    deviance = rep(NA, I)
    for (i in 1:I) {
        pMult = dbinom(c(1:J[i]), size=J[i], prob=pVec[i])
        ysumMean = sum(lambdaVec[i]*pMult)
        deviance[i] = ysumMean - ysumArg[i] + ifelse(ysumArg[i]==0, 0, ysumArg[i]*log(ysumArg[i]/ysumMean))
    }
    2*sum(deviance)
}


gof.MultinomPoissonMixture = function(devianceArg, n.gof, param, xArg, wArg, Jarg) {
    beta = param[1:dim(xArg)[2]]
    alpha = param[(1+dim(xArg)[2]):(dim(xArg)[2]+dim(wArg)[2])]
    
    pVec =  expit(wArg %*% alpha)
    lambdaVec = as.vector(exp(xArg %*% beta))
    I = length(Jarg)
    pMult = matrix(nrow=I, ncol=max(Jarg)+1)
    for (i in 1:I) {
        pMult[i,1:(J[i]+1)] = dbinom(c(1:J[i], 0), size=J[i], prob=pVec[i])
    }

    deviance = rep(NA, n.gof)
    for (i.gof in 1:n.gof) {
        
        # simulate counts
        y = matrix(nrow=I, ncol=max(J))
        N = rpois(I, lambda=lambdaVec)
        for (i in 1:I) {
            y[i,1:J[i]] = t(rmultinom(1, size=N[i], prob=pMult[i,1:(J[i]+1)]))[1:J[i]]
        }

        # fit N-mixture model to simulated counts
        paramGuess = param
        fit = optim(par=paramGuess, fn=negLL.MultinomPoissonMixture, method='BFGS', yArg=y, xArg=xArg, wArg=wArg, Jarg=Jarg)

        # compute deviance for simulated counts
        deviance[i.gof] = deviance.MultinomPoissonMixture(fit$par, ysumArg=apply(y,1,sum,na.rm=TRUE), xArg=xArg, wArg=wArg, Jarg=Jarg)

    }
    tailProb =  mean(deviance > devianceArg)
    gof.est = tailProb
    gof.lower = tailProb - qnorm(.975) * sqrt(tailProb * (1-tailProb) / n.gof)
    gof.upper = tailProb + qnorm(.975) * sqrt(tailProb * (1-tailProb) / n.gof)
    retVal = c(gof.est, gof.lower, gof.upper)
    retVal
}


CR = '\n'
beta.est = matrix(nrow=nrow(xmatCol), ncol=K)
alpha.est = matrix(nrow=nrow(wmatCol), ncol=K)
deviance.est = rep(NA,K)
gof.est = matrix(nrow=3, ncol=K)
dimnames(gof.est)[[1]] = c('gof','gof 2.5%','gof 97.5%')
mu.est = matrix(nrow=I, ncol=K)
for (k in 1:K) {   
    xcol = xmatCol[1:nx[k], k]
    X = matrix(xmat[,xcol], ncol=nx[k])
    beta.est[1:nx[k], k] = glm(ysum[,k]~X-1, family=poisson)$coefficients

    wcol = wmatCol[1:nw[k], k]
    W = matrix(wmat[,wcol], ncol=nw[k])
    alpha.est[1:nw[k], k] = 0

    paramGuess = c(as.vector(beta.est[1:nx[k],k]), as.vector(alpha.est[1:nw[k],k]))
    fit = optim(par=paramGuess, fn=negLL.MultinomPoissonMixture, method='BFGS', yArg=y[,k,], xArg=X, wArg=W, Jarg=J)

    beta.est[1:nx[k], k] = fit$par[1:nx[k]]
    alpha.est[1:nw[k], k] = fit$par[(nx[k]+1):(nx[k]+nw[k])]
    
    deviance.est[k] = deviance.MultinomPoissonMixture(fit$par, ysumArg=ysum[,k], xArg=X, wArg=W, Jarg=J)
    ## gof.est[,k] = gof.MultinomPoissonMixture(deviance.est[k], 500, fit$par, xArg=X, wArg=W, Jarg=J)
    mu.est[, k] = X %*% beta.est[1:nx[k], k]
    cat('Species ', species.Number[species.ind][k], ':  deviance = ', deviance.est[k], '  GOF = ', gof.est[,k], CR)
}

eps.est = log(ysum+1) - mu.est
Sigma.est = var(eps.est)





# Use Gibbs sampling algorithm to estimate summaries of the posterior distribution

# .... define functions used in Metropolis-Hastings sampling

logDensity.eps = function(epsArg, Karg, nArg, muArg, SigmaArg) {
    lambdaVec = exp(muArg + epsArg)
    retVal = dmvnorm(epsArg, mean=rep(0,Karg), sigma=SigmaArg, log=TRUE) + sum(dpois(nArg, lambda=lambdaVec, log=TRUE))
    retVal
}

negGrad.eps = function(epsArg, Karg, nArg, muArg, SigmaArg) {
    SigmaInv = chol2inv(chol(SigmaArg))
    lambdaVec = exp(muArg + epsArg)
    retVal = SigmaInv %*% epsArg - nArg + lambdaVec
    retVal
}
    
negHess.eps = function(epsArg, muArg, SigmaInvArg) {
    lambdaVec = exp(muArg + epsArg)
    retVal = SigmaInvArg  + diag(lambdaVec)
    retVal
}

neglogDensity.eps = function(epsArg, Karg, nArg, muArg, SigmaArg) {
    (-1)*logDensity.eps(epsArg, Karg, nArg, muArg, SigmaArg)
}



logPrior.beta = function(betaArg, mu.beta, sigma.beta) {
    p = length(betaArg)
    beta.mean = rep(mu.beta, p)
    ones = rep(1, p)
    beta.cov = sigma.beta*diag(ones)
    dmvnorm(betaArg, mean=beta.mean, sigma=beta.cov, log=TRUE)
}

logDensity.beta = function(betaArg, nArg, xArg, epsArg, mu.beta, sigma.beta) {
    lambdaVec = exp(xArg %*% betaArg + epsArg)
    retVal = logPrior.beta(betaArg, mu.beta, sigma.beta) + sum(dpois(nArg, lambda=lambdaVec, log=TRUE))
    retVal
}

neglogDensity.beta = function(betaArg, nArg, xArg, epsArg, mu.beta, sigma.beta) {
    (-1)*logDensity.beta(betaArg, nArg, xArg, epsArg, mu.beta, sigma.beta)
}



logPrior.alpha = function(alphaArg, mu.alpha, sigma.alpha, nu.alpha) {
    p = length(alphaArg)
    alpha.mean = rep(mu.alpha, p)
    ones = rep(1, p)
    alpha.cov = sigma.alpha*diag(ones)
    dmvt(alphaArg, delta=alpha.mean, sigma=alpha.cov, df=nu.alpha, log=TRUE)
}

logDensity.alpha = function(alphaArg, yArg, ysumArg, nArg, wArg, Jarg, mu.alpha, sigma.alpha, nu.alpha) {
    pVec =  expit(wArg %*% alphaArg)
    retVal = logPrior.alpha(alphaArg, mu.alpha, sigma.alpha, nu.alpha)
    for (i in 1:length(nArg)) {
        pMult = dbinom(c(1:J[i], 0), size=J[i], prob=pVec[i])
        retVal = retVal + dmultinom(c(yArg[i,1:J[i]], nArg[i]-ysumArg[i]), size=nArg[i], prob=pMult, log=TRUE) 
    }
    retVal
}

negGrad.alpha  = function(alphaArg, yArg, ysumArg, nArg, wArg, Jarg, mu.alpha, sigma.alpha, nu.alpha) {
    
    term1 = (2* alphaArg * (1 + 1/nu.alpha) ) / (sigma.alpha^2 * (1 + (1/nu.alpha)*(alphaArg/sigma.alpha)^2 ) )

    
    pVec =  expit(wArg %*% alphaArg)
    tempVec = nArg*Jarg*pVec
    tempSum1 = t(wArg) %*% tempVec

    I = dim(yArg)[1]
    jysum = rep(NA,length=I)
    for (i in 1:I) {
        jysum[i] = sum( c(1:J[i]) * yArg[i,1:J[i]] )
    }
    tempVec = jysum
    tempSum2 = t(wArg) %*% tempVec
   
    term2 = as.vector(tempSum1 - tempSum2)
        
    retVal = term1 + term2
    retVal
}


neglogDensity.alpha = function(alphaArg, yArg, ysumArg, nArg, wArg, Jarg, mu.alpha, sigma.alpha, nu.alpha) {
    (-1)*logDensity.alpha(alphaArg, yArg, ysumArg, nArg, wArg, Jarg, mu.alpha, sigma.alpha, nu.alpha)
}




# .... assign values for adaptive Metropolis-Hastings sampling
nuOfProposal.eps = rep(2.0, I)
naccept.eps = rep(0, I)
nuOfProposal.beta = rep(2.0, K)
naccept.beta = rep(0, K)
nuOfProposal.alpha = rep(2.0, K)
naccept.alpha = rep(0, K)


# ... assign prior parameter values
mu.beta = 0
sigma.beta = 100
mu.alpha = 0
sigma.alpha = 1.566
nu.alpha = 7.763
nu.Sigma = 2
scale.b = rep(1.0, K)


# .... initialize Markov chain
beta = beta.est
Sigma = Sigma.est
SigmaInv = chol2inv(chol(Sigma))
eps = eps.est
N = ysum
alpha = alpha.est



# .... compute Gibbs draws

continueGibbs = TRUE
draw = 0
ndraws = 170000
CR = '\n'
cat('Begin Gibbs sampling:', CR, CR)


beta.ind = !is.na(xmatCol)  # index for elements of beta matrix to save
Sigma.ind = lower.tri(Sigma, diag=TRUE)  # index for elements of Sigma matrix to save
alpha.ind = !is.na(wmatCol)  # index for elements of alpha matrix to save

cat(as.vector(beta[beta.ind]), sep=',', file='markovchainForBeta.txt')
cat(CR, file='markovchainForBeta.txt', append=TRUE)

cat(Sigma[Sigma.ind], sep=',', file='markovchainForSigma.txt')
cat(CR, file='markovchainForSigma.txt', append=TRUE)

cat(as.vector(alpha[alpha.ind]), sep=',', file='markovchainForAlpha.txt')
cat(CR, file='markovchainForAlpha.txt', append=TRUE)


start.time = Sys.time()

while(continueGibbs) {
  
  draw = draw + 1
  drawinterval = 100
  if (draw == round(draw/drawinterval)*drawinterval)  {
      end.time = Sys.time()
      elapsed.time = difftime(end.time, start.time, units='mins')
      cat('..... drawing sample #', draw, ' after ', elapsed.time, ' minutes', CR)
  }


  # draw eps (by site)
  # ... first update linear predictor at all sites using current value of beta
  mu = matrix(nrow=I, ncol=K)
  for (k in 1:K) {   
    xcol = xmatCol[1:nx[k], k]
    X = matrix(xmat[,xcol], ncol=nx[k])
    mu[ , k] = X %*% beta[1:nx[k], k]
  }
  # ... now draw eps by site
  SigmaInv = chol2inv(chol(Sigma))
  for (i in 1:I) {
      # find mode and hessian of unnormalized conditional density function
      fit = optim(par=eps[i,], fn=neglogDensity.eps, gr=negGrad.eps, method='BFGS', hessian=FALSE, Karg=K, nArg=N[i,], muArg=mu[i,], SigmaArg=Sigma)
      
      # draw candidate using multivariate t distribution as proposal
      meanOfProposal.eps = fit$par
      fit.hessian = negHess.eps(fit$par, muArg=mu[i,], SigmaInvArg=SigmaInv)
      SigmaOfProposal.eps = chol2inv(chol(fit.hessian))
      
      
      eps.cand = as.vector(rmvt(1, delta=meanOfProposal.eps, sigma=SigmaOfProposal.eps, df=nuOfProposal.eps[i], type='shifted'))

      logR = logDensity.eps(eps.cand, Karg=K, nArg=N[i,], muArg=mu[i,], SigmaArg=Sigma) - dmvt(eps.cand, delta=meanOfProposal.eps, sigma=SigmaOfProposal.eps, df=nuOfProposal.eps[i], log=TRUE) - logDensity.eps(eps[i,], Karg=K, nArg=N[i,], muArg=mu[i,], SigmaArg=Sigma) + dmvt(eps[i,], delta=meanOfProposal.eps, sigma=SigmaOfProposal.eps, df=nuOfProposal.eps[i], log=TRUE)
      if (logR >= 0) {
          eps[i,] = eps.cand
          naccept.eps[i] = naccept.eps[i] + 1
      }
      else {
          if (runif(1,0,1) <= exp(logR)) {
              eps[i,] = eps.cand
              naccept.eps[i] = naccept.eps[i] + 1
          }
      }      
  }
  


  # draw b; then draw inverse of Sigma
  b = rgamma(K, shape=(nu.Sigma+K)/2, rate=(1/scale.b^2) + nu.Sigma*diag(SigmaInv))
  crossProdMat = t(eps) %*% eps
  SigmaInv = rWishart(1, df=nu.Sigma+I+K-1,  Sigma=chol2inv(chol(2*nu.Sigma*diag(b) + crossProdMat)))[,,1]
  Sigma = chol2inv(chol(SigmaInv))



  # draw beta (by species)
  for (k in 1:K) {
      # find mode and hessian of unnormalized conditional density function
      xcol = xmatCol[1:nx[k], k]
      X = matrix(xmat[,xcol], ncol=nx[k])
      fit = optim(par=beta[1:nx[k],k], fn=neglogDensity.beta, method='BFGS', hessian=TRUE, nArg=N[,k], xArg=X, epsArg=eps[,k], mu.beta=mu.beta, sigma.beta=sigma.beta)

      
      # draw candidate using multivariate t distribution as proposal
      meanOfProposal.beta = fit$par
      SigmaOfProposal.beta = chol2inv(chol(fit$hessian))
      beta.cand = as.vector(rmvt(1, delta=meanOfProposal.beta, sigma=SigmaOfProposal.beta, df=nuOfProposal.beta[k], type='shifted'))

      logR = logDensity.beta(beta.cand, nArg=N[,k], xArg=X, epsArg=eps[,k], mu.beta=mu.beta, sigma.beta=sigma.beta) - dmvt(beta.cand, delta=meanOfProposal.beta, sigma=SigmaOfProposal.beta, df=nuOfProposal.beta[k], log=TRUE) - logDensity.beta(beta[1:nx[k],k], nArg=N[,k], xArg=X, epsArg=eps[,k], mu.beta=mu.beta, sigma.beta=sigma.beta) + dmvt(beta[1:nx[k],k], delta=meanOfProposal.beta, sigma=SigmaOfProposal.beta, df=nuOfProposal.beta[k], log=TRUE)
      if (logR >= 0) {
          beta[1:nx[k],k] = beta.cand
          naccept.beta[k] = naccept.beta[k] + 1
      }
      else {
          if (runif(1,0,1) <= exp(logR)) {
              beta[1:nx[k],k] = beta.cand
              naccept.beta[k] = naccept.beta[k] + 1
          }
      }      
  }


  # draw N
  for (k in 1:K) {
      xcol = xmatCol[1:nx[k], k]
      X = matrix(xmat[,xcol], ncol=nx[k])
      muVec = X %*% beta[1:nx[k], k]
      lambdaVec = exp(muVec + eps[,k])
      wcol = wmatCol[1:nw[k], k]
      W = matrix(wmat[,wcol], ncol=nw[k])
      etaVec = W %*% alpha[1:nw[k], k]
      pVec = expit(etaVec)
      N[,k] = ysum[,k] + rpois(I, lambda=lambdaVec*(1-pVec)^J)
  }


  # draw alpha (by species)
  for (k in 1:K) {
      # find mode and hessian of unnormalized conditional density function
      wcol = wmatCol[1:nw[k], k]
      W = matrix(wmat[,wcol], ncol=nw[k])
      fit = optim(par=alpha[1:nw[k],k], fn=neglogDensity.alpha, gr=negGrad.alpha, method='BFGS', hessian=TRUE, yArg=y[,k,], ysumArg=ysum[,k], nArg=N[,k], wArg=W, Jarg=J, mu.alpha=mu.alpha, sigma.alpha=sigma.alpha, nu.alpha=nu.alpha)
      
      # draw candidate using multivariate t distribution as proposal
      meanOfProposal.alpha = fit$par
      SigmaOfProposal.alpha = chol2inv(chol(fit$hessian))
      alpha.cand = as.vector(rmvt(1, delta=meanOfProposal.alpha, sigma=SigmaOfProposal.alpha, df=nuOfProposal.alpha[k], type='shifted'))

      logR = logDensity.alpha(alpha.cand, yArg=y[,k,], ysumArg=ysum[,k], nArg=N[,k], wArg=W, Jarg=J, mu.alpha=mu.alpha, sigma.alpha=sigma.alpha, nu.alpha=nu.alpha) - dmvt(alpha.cand, delta=meanOfProposal.alpha, sigma=SigmaOfProposal.alpha, df=nuOfProposal.alpha[k], log=TRUE) - logDensity.alpha(alpha[1:nw[k],k], yArg=y[,k,], ysumArg=ysum[,k], nArg=N[,k], wArg=W, Jarg=J, mu.alpha=mu.alpha, sigma.alpha=sigma.alpha, nu.alpha=nu.alpha) + dmvt(alpha[1:nw[k],k], delta=meanOfProposal.alpha, sigma=SigmaOfProposal.alpha, df=nuOfProposal.alpha[k], log=TRUE)
      if (logR >= 0) {
          alpha[1:nw[k],k] = alpha.cand
          naccept.alpha[k] = naccept.alpha[k] + 1
      }
      else {
          if (runif(1,0,1) <= exp(logR)) {
              alpha[1:nw[k],k] = alpha.cand
              naccept.alpha[k] = naccept.alpha[k] + 1
          }
      }
  }


  
  # write draw to file
  cat(Sigma[Sigma.ind], sep=',', file='markovchainForSigma.txt', append=TRUE)
  cat(CR, file='markovchainForSigma.txt', append=TRUE)

  cat(as.vector(beta[beta.ind]), sep=',', file='markovchainForBeta.txt', append=TRUE)
  cat(CR, file='markovchainForBeta.txt', append=TRUE)
  
  cat(as.vector(alpha[alpha.ind]), sep=',', file='markovchainForAlpha.txt', append=TRUE)
  cat(CR, file='markovchainForAlpha.txt', append=TRUE)

  if (draw == ndraws) {
    cat('Completed ', ndraws, ' draws of MCMC algorithm', CR)
    numOfDraws = as.integer(readline(prompt='Enter additional number of MCMC draws -> '))

    if (numOfDraws == 0) {
      continueGibbs = FALSE
    }
    else {
      ndraws = ndraws + numOfDraws
    }
  }

}  # end of while loop




# create vector of species numbers used in analysis
speciesNumber = species.Number[species.ind]

#  read saved elements of Sigma from file; reconstruct Sigma; compute std deviation and correlation parameters
mc.Sigma = as.matrix(read.csv('markovchainForSigma.txt', header=F))

ind.post = 20001:170000
mc.Sigma = mc.Sigma[ind.post, ]
ndraws = length(ind.post)
mc.sigma = matrix(nrow=ndraws, ncol=K)
mc.rho = matrix(nrow=ndraws, ncol=K*(K-1)/2 + K)

for (draw in 1:ndraws) {
    Sigma.lower = LowerTriangle(mc.Sigma[draw, ], K)
    Sigma.upper = t(Sigma.lower)
    diag(Sigma.upper) = 0
    Sigma = Sigma.lower + Sigma.upper
    sigma = sqrt(diag(Sigma))
    sigmaInv = 1/sigma
    sigmaInvMat = diag(sigmaInv)
    Rho = sigmaInvMat %*% Sigma %*% sigmaInvMat
    mc.sigma[draw, ] = sigma
    rho.ind = lower.tri(Rho, diag=TRUE)
    mc.rho[draw, ] = Rho[rho.ind]
}

# Compute posterior medians and quantiles for Rho
rhoPost.hpdinterval = HPDinterval(mcmc(mc.rho), prob=0.95)
rhoPost.median = LowerTriangle(apply(mc.rho,2,median), K)
dimnames(rhoPost.median) = list(speciesNumber, speciesNumber)
rhoPost.lower = LowerTriangle(rhoPost.hpdinterval[,'lower'], K)
rhoPost.upper = LowerTriangle(rhoPost.hpdinterval[,'upper'], K)
rhoPost.nonzeroInd = (rhoPost.lower<0 & rhoPost.upper<0) | (rhoPost.lower>0 & rhoPost.upper>0)
diag(rhoPost.nonzeroInd) = FALSE
dimnames(rhoPost.nonzeroInd) = list(speciesNumber, speciesNumber)

# ... compute Monte Carlo SE of posterior stats
rhoPost.median.MCSE = rep(NA, dim(mc.rho)[2])
for (i in 1:dim(mc.rho)[2]) {
  postVec = as.vector(mc.rho[,i])
  rhoPost.median.MCSE[i] = mcse(postVec, median)$se
}
rhoPost.median.MCSE = LowerTriangle(rhoPost.median.MCSE, K)
dimnames(rhoPost.median.MCSE) = list(speciesNumber, speciesNumber)


# Compute posterior medians and quantiles for sigma
prob.quantiles = c(.025, .975)  # for credible limits
sigmaPost.median = apply(mc.sigma,2,median)
sigmaPost.hpdinterval = HPDinterval(mcmc(mc.sigma), prob=prob.quantiles[2]-prob.quantiles[1])
sigmaPost.stats = cbind(sigmaPost.median, sigmaPost.hpdinterval)

prob.names = paste(as.character(100*prob.quantiles), '%', sep='')
dimnames(sigmaPost.stats)[[1]] = speciesNumber
dimnames(sigmaPost.stats)[[2]] = c('Median', prob.names)

# ... compute Monte Carlo SE of posterior stats
sigmaPost.stats.MCSE = sigmaPost.stats
for (i in 1:dim(sigmaPost.stats)[1]) {
  postVec = as.vector(mc.sigma[,i])
  sigmaPost.stats.MCSE[i,1] = mcse(postVec, median)$se
  sigmaPost.stats.MCSE[i,2] = mcse(postVec, lowerQuantile)$se
  sigmaPost.stats.MCSE[i,3] = mcse(postVec, upperQuantile)$se
}



# Compute posterior means and quantiles for beta

mc.beta = as.matrix(read.csv('markovchainForBeta.txt', header=F))
mc.beta = mc.beta[ind.post, ]

postvec = apply(mc.beta,2,mean)
betaPost.mean = matrix(nrow=length(nx), ncol=dim(xmat)[2])
dimnames(betaPost.mean) = list(speciesNumber, dimnames(xmat)[[2]])
cumsumnx = cumsum(nx)
for (i in 1:length(nx)) {
    xcol = xmatCol[1:nx[i], i]
    if (i>1) {
        betaPost.mean[i,xcol] = postvec[(cumsumnx[i-1]+1):cumsumnx[i]]
    }
    else {
        betaPost.mean[i,xcol] = postvec[1:nx[i]]
    }
}

postvec = apply(mc.beta,2,lowerQuantile)
betaPost.lower = betaPost.mean
for (i in 1:length(nx)) {
    xcol = xmatCol[1:nx[i], i]
    if (i>1) {
        betaPost.lower[i,xcol] = postvec[(cumsumnx[i-1]+1):cumsumnx[i]]
    }
    else {
        betaPost.lower[i,xcol] = postvec[1:nx[i]]
    }
}
postvec = apply(mc.beta,2,upperQuantile)
betaPost.upper = betaPost.mean
for (i in 1:length(nx)) {
    xcol = xmatCol[1:nx[i], i]
    if (i>1) {
        betaPost.upper[i,xcol] = postvec[(cumsumnx[i-1]+1):cumsumnx[i]]
    }
    else {
        betaPost.upper[i,xcol] = postvec[1:nx[i]]
    }
}

betaPost.nonzeroInd = (betaPost.lower<0 & betaPost.upper<0) | (betaPost.lower>0 & betaPost.upper>0)




# Compute posterior means and quantiles for alpha

mc.alpha = as.matrix(read.csv('markovchainForAlpha.txt', header=F))
mc.alpha = mc.alpha[ind.post, ]

postvec = apply(mc.alpha,2,mean)
alphaPost.mean = matrix(nrow=length(nw), ncol=dim(wmat)[2])
dimnames(alphaPost.mean) = list(speciesNumber, dimnames(wmat)[[2]])
cumsumnw = cumsum(nw)
for (i in 1:length(nw)) {
    wcol = wmatCol[1:nw[i], i]
    if (i>1) {
        alphaPost.mean[i,wcol] = postvec[(cumsumnw[i-1]+1):cumsumnw[i]]
    }
    else {
        alphaPost.mean[i,wcol] = postvec[1:nw[i]]
    }
}

postvec = apply(mc.alpha,2,lowerQuantile)
alphaPost.lower = alphaPost.mean
for (i in 1:length(nw)) {
    wcol = wmatCol[1:nw[i], i]
    if (i>1) {
        alphaPost.lower[i,wcol] = postvec[(cumsumnw[i-1]+1):cumsumnw[i]]
    }
    else {
        alphaPost.lower[i,wcol] = postvec[1:nw[i]]
    }
}
postvec = apply(mc.alpha,2,upperQuantile)
alphaPost.upper = alphaPost.mean
for (i in 1:length(nw)) {
    wcol = wmatCol[1:nw[i], i]
    if (i>1) {
        alphaPost.upper[i,wcol] = postvec[(cumsumnw[i-1]+1):cumsumnw[i]]
    }
    else {
        alphaPost.upper[i,wcol] = postvec[1:nw[i]]
    }
}

alphaPost.nonzeroInd = (alphaPost.lower<0 & alphaPost.upper<0) | (alphaPost.lower>0 & alphaPost.upper>0)



# Tabulate MLEs  using Askins' ordering of species
d.species = read.csv('Table_with_species_list.csv')
outInd = match(d.species[,'Species.No.'], speciesNumber)
outInd = outInd[!is.na(outInd)]

speciesNumber[outInd]

round(betaPost.mean[outInd,], 2)

betaPost.nonzeroInd[outInd,]

round(alphaPost.mean[outInd,], 2)

alphaPost.nonzeroInd[outInd,]


