# FITS SINGLE SPECIES N-MIXTURE MODELS USING METHOD OF MAXIMUM LIKELIHOOD


# define utility functions
logit = function(xx) { log(xx) - log(1-xx) }
expit = function(xx) { 1/(1 + exp(-xx)) }


lowerQuantile = function(z) {  quantile(z, probs=.025)  }
upperQuantile = function(z) {  quantile(z, probs=.975)  }




# read detection data from files
d = read.csv('Askins-period-new-observer-1-27-14.csv')
d = d[!is.na(d[,'period']), ]

site = as.integer(d[,'point'])
date = as.character(d[,'date'])
observer = as.character(d[,'observer'])
weather = as.character(d[,'weather'])
timeOfDay = as.integer(d[,'time'])
timePeriod = as.character(d[,'time2'])


# extract data for unique sampling events
survey = unique(data.frame(site,date,observer,weather,timeOfDay,timePeriod))
dimnames(survey)[[1]] = 1:dim(survey)[1]
I = dim(survey)[1]
J = rep(4, I)

# compute year and day of year from date
dateVec = as.Date(survey[,'date'], format='%m/%d/%Y')
year = substr(dateVec, 1,4)
yrIndex = (year=='1983')
dayOfYear = rep(NA, I)
dayOfYear[yrIndex] = julian(dateVec[yrIndex], origin=as.Date('1983-01-10'))
dayOfYear[!yrIndex] = julian(dateVec[!yrIndex], origin=as.Date('1984-01-10'))
survey = cbind(survey, year, dayOfYear)

# compute species-specific counts for each sampling event
species.Number = sort(unique(d[,'species.num']))
K = length(species.Number)

y = array(0, dim=c(I,K,max(J)))

for (i in 1:I) {
    ind = d[,'point']==survey[i,'site'] & as.character(d[,'date'])==as.character(survey[i,'date']) & as.character(d[,'observer'])==as.character(survey[i,'observer']) &  as.character(d[,'weather'])==as.character(survey[i,'weather']) & as.integer(d[,'time'])==as.integer(survey[i,'timeOfDay']) & as.character(d[,'time2'])==as.character(survey[i,'timePeriod'])
    
    dsub = d[ind, c('species.num','individual')]

    for (k in 1:K) {
        ind.species = dsub[,'species.num']==species.Number[k]
        if (sum(ind.species)>0) {
            individual.species = dsub[ind.species,'individual']
            nindividuals = table(table(individual.species))
            nperiods = as.integer(names(nindividuals))
            y[i,k, nperiods] = nindividuals
        }
    }
}

# compute total number of individuals detected for each sampling event and species
ysum = matrix(nrow=I, ncol=K)
for (i in 1:I) {
    ysum[i, ] = t(apply(y[i,,], 1, sum, na.rm=TRUE))
}


# read in site-specific data (e.g., habitat covariate measurements);
# then merge with sampling-event data
d = read.csv('Askins-by-point-wcov-11-5-12.csv')
colInd = c(1,4:19)
hab = d[1, colInd]
for (i in 1:I) {
    rowInd = d[,'point']==survey[i,'site']
    if (i==1) {
        hab[1,] = d[rowInd, colInd]
    }
    else {
        hab = rbind(hab, d[rowInd, colInd])
    }
}
survey = cbind(survey, hab)




# construct covariate matrices for analysis

landVars = c('area..ha.', 'donut.area', 'isolat..km.', 'regional.forest.area')
land = survey[, landVars]
land.pca = prcomp(land, center=TRUE, scale=TRUE, retx=TRUE)
landPC1 = land.pca$x[,1]
landPC2 = land.pca$x[,2]

vegVars = c('ave.canopy.height', 'basal.area.dead.trees', 'basal.area.trees', 'number.herb.species', 'number.tree.species', 'percent..cover.trees', 'percent.cover.herbs', 'percent.cover.shrubs', 'relative.dominance.conifers', 'species.diversity.trees.Shannon', 'tree.density')
veg = survey[, vegVars]
veg.pca = prcomp(veg, center=TRUE, scale=TRUE, retx=TRUE)
vegPC1 = veg.pca$x[,1]
vegPC2 = veg.pca$x[,2]
vegPC3 = veg.pca$x[,3]
vegPC4 = veg.pca$x[,4]
vegPC5 = veg.pca$x[,5]
vegPC6 = veg.pca$x[,6]


# ... assign potential species-specific covariates of abundance to xmat
oneVec = matrix(1,nrow=I,ncol=1)
xmat = cbind(oneVec, landPC1, landPC2, vegPC1, vegPC2, vegPC3, vegPC4, vegPC5, vegPC6)
dimnames(xmat)[[2]][1] = 'Intercept'

# ... assign potential species-specific covariates of detection to wmat
doy = (survey[,'dayOfYear'] - mean(survey[,'dayOfYear'])) / sd(survey[,'dayOfYear'])
survey = cbind(survey, doy)
wmat = model.matrix(~timePeriod+doy, data=survey)


# ... for each of K species, assign columns of xmat to be included in the model
xmatCol = matrix(nrow=5, ncol=K) 
xmatCol[1:1, 1] = c(1)
xmatCol[1:2, 2] = c(1,4)
xmatCol[1:2, 3] = c(1,5)
xmatCol[1:3, 4] = c(1,2,3)
xmatCol[1:4, 5] = c(1,2,3,5)
xmatCol[1:1, 6] = c(1)
xmatCol[1:2, 7] = c(1,4)
xmatCol[1:1, 8] = c(1)
xmatCol[1:1, 9] = c(1)
xmatCol[1:1, 10] = c(1)
xmatCol[1:2, 11] = c(1,2)
xmatCol[1:1, 12] = c(1)
xmatCol[1:3, 13] = c(1,2,5)
xmatCol[1:1, 14] = c(1)
xmatCol[1:1, 15] = c(1)
xmatCol[1:1, 16] = c(1)
xmatCol[1:1, 17] = c(1)
xmatCol[1:1, 18] = c(1)
xmatCol[1:1, 19] = c(1)
xmatCol[1:2, 20] = c(1,2)
xmatCol[1:1, 21] = c(1)
xmatCol[1:2, 22] = c(1,2)
xmatCol[1:1, 23] = c(1)
xmatCol[1:1, 24] = c(1)
xmatCol[1:1, 25] = c(1)
xmatCol[1:1, 26] = c(1)
xmatCol[1:3, 27] = c(1,5,6)
xmatCol[1:2, 28] = c(1,2)
xmatCol[1:2, 29] = c(1,2)
xmatCol[1:2, 30] = c(1,2)
xmatCol[1:2, 31] = c(1,2)
xmatCol[1:1, 32] = c(1)
xmatCol[1:1, 33] = c(1)
xmatCol[1:2, 34] = c(1,2)
xmatCol[1:4, 35] = c(1,2,3,5)
xmatCol[1:3, 36] = c(1,3,5)
xmatCol[1:2, 37] = c(1,2)
xmatCol[1:2, 38] = c(1,2)
xmatCol[1:2, 39] = c(1,2)
xmatCol[1:2, 40] = c(1,2)
xmatCol[1:4, 41] = c(1,2,4,5)
xmatCol[1:2, 42] = c(1,4)
xmatCol[1:2, 43] = c(1,2)
xmatCol[1:3, 44] = c(1,2,5)
xmatCol[1:3, 45] = c(1,2,3)
xmatCol[1:2, 46] = c(1,4)
xmatCol[1:3, 47] = c(1,2,5)
xmatCol[1:1, 48] = c(1)
xmatCol[1:2, 49] = c(1,2)
xmatCol[1:4, 50] = c(1,2,3,5)
xmatCol[1:1, 51] = c(1)
xmatCol[1:5, 52] = c(1,2,4,5,7)
xmatCol[1:3, 53] = c(1,2,7)
xmatCol[1:1, 54] = c(1)
xmatCol[1:1, 55] = c(1)
xmatCol[1:3, 56] = c(1,2,3)
xmatCol[1:1, 57] = c(1)
xmatCol[1:2, 58] = c(1,2)
xmatCol[1:1, 59] = c(1)
xmatCol[1:2, 60] = c(1,2)
xmatCol[1:2, 61] = c(1,2)
xmatCol[1:1, 62] = c(1)
xmatCol[1:1, 63] = c(1)
xmatCol[1:1, 64] = c(1)
xmatCol[1:2, 65] = c(1,4)
xmatCol[1:1, 66] = c(1)
xmatCol[1:1, 67] = c(1)
xmatCol[1:3, 68] = c(1,2,3)
xmatCol[1:1, 69] = c(1)
xmatCol[1:1, 70] = c(1)
xmatCol[1:1, 71] = c(1)
xmatCol[1:1, 72] = c(1)
xmatCol[1:1, 73] = c(1)

nx = apply(!is.na(xmatCol), 2, sum)  # number of regressors of abundance for each species



# ... for each of K species, assign columns of wmat to be included in the model
wmatCol = matrix(nrow=4, ncol=K)
wmatCol[1, ] = c(1)
wmatCol[2:2, c(8,14,22,23,33,44,47,48,51,55,59)] = c(4)

nw = apply(!is.na(wmatCol), 2, sum)  # number of regressors of detection probability for each species



# ... assign species to be included in the model
## species.ind = match(c(22,28,32,39,42,50,54,57), species.Number)
species.ind = rep(TRUE, K)
y = y[, species.ind, ]
ysum = ysum[, species.ind]
K = dim(ysum)[2]

xmatCol = xmatCol[, species.ind]
if (!is.matrix(xmatCol)) xmatCol = matrix(xmatCol, ncol=K)  # check if all species have only intercept
nx = nx[species.ind]
wmatCol = wmatCol[, species.ind]
if (!is.matrix(wmatCol)) wmatCol = matrix(wmatCol, ncol=K)  # check if all species have only intercept
nw = nw[species.ind]




# fit model

# ... compute parameter estimates using single-species N-mixture models

negLL.MultinomPoissonMixture = function(param, yArg, xArg, wArg, Jarg) {
    beta = param[1:dim(xArg)[2]]
    alpha = param[(1+dim(xArg)[2]):(dim(xArg)[2]+dim(wArg)[2])]
    
    pVec =  expit(wArg %*% alpha)
    lambdaVec = exp(xArg %*% beta)
    I = dim(yArg)[1]

    logL = rep(NA, I)
    for (i in 1:I) {
        pMult = dbinom(c(1:J[i]), size=J[i], prob=pVec[i])
        logL[i] = sum(dpois(yArg[i,1:J[i]], lambda=lambdaVec[i]*pMult, log=TRUE))
    }
    (-1)*sum(logL)
}


deviance.MultinomPoissonMixture = function(param, ysumArg, xArg, wArg, Jarg) {
    beta = param[1:dim(xArg)[2]]
    alpha = param[(1+dim(xArg)[2]):(dim(xArg)[2]+dim(wArg)[2])]
    
    pVec =  expit(wArg %*% alpha)
    lambdaVec = exp(xArg %*% beta)
    I = length(ysumArg)

    deviance = rep(NA, I)
    for (i in 1:I) {
        pMult = dbinom(c(1:J[i]), size=J[i], prob=pVec[i])
        ysumMean = sum(lambdaVec[i]*pMult)
        deviance[i] = ysumMean - ysumArg[i] + ifelse(ysumArg[i]==0, 0, ysumArg[i]*log(ysumArg[i]/ysumMean))
    }
    2*sum(deviance)
}


gof.MultinomPoissonMixture = function(devianceArg, n.gof, param, xArg, wArg, Jarg) {
    beta = param[1:dim(xArg)[2]]
    alpha = param[(1+dim(xArg)[2]):(dim(xArg)[2]+dim(wArg)[2])]
    
    pVec =  expit(wArg %*% alpha)
    lambdaVec = as.vector(exp(xArg %*% beta))
    I = length(Jarg)
    pMult = matrix(nrow=I, ncol=max(Jarg)+1)
    for (i in 1:I) {
        pMult[i,1:(J[i]+1)] = dbinom(c(1:J[i], 0), size=J[i], prob=pVec[i])
    }

    deviance = rep(NA, n.gof)
    for (i.gof in 1:n.gof) {
        
        # simulate counts
        y = matrix(nrow=I, ncol=max(J))
        N = rpois(I, lambda=lambdaVec)
        for (i in 1:I) {
            y[i,1:J[i]] = t(rmultinom(1, size=N[i], prob=pMult[i,1:(J[i]+1)]))[1:J[i]]
        }

        # fit N-mixture model to simulated counts
        paramGuess = param
        fit = optim(par=paramGuess, fn=negLL.MultinomPoissonMixture, method='BFGS', yArg=y, xArg=xArg, wArg=wArg, Jarg=Jarg)

        # compute deviance for simulated counts
        deviance[i.gof] = deviance.MultinomPoissonMixture(fit$par, ysumArg=apply(y,1,sum,na.rm=TRUE), xArg=xArg, wArg=wArg, Jarg=Jarg)

    }
    tailProb =  mean(deviance > devianceArg)
    gof.est = tailProb
    gof.lower = tailProb - qnorm(.975) * sqrt(tailProb * (1-tailProb) / n.gof)
    gof.upper = tailProb + qnorm(.975) * sqrt(tailProb * (1-tailProb) / n.gof)
    retVal = c(gof.est, gof.lower, gof.upper)
    retVal
}



CR = '\n'
beta.est = matrix(nrow=nrow(xmatCol), ncol=K)
alpha.est = matrix(nrow=nrow(wmatCol), ncol=K)
beta.lowerCL = beta.est
beta.upperCL = beta.est
alpha.lowerCL = alpha.est
alpha.upperCL = alpha.est
aic.est = rep(NA,K)
bic.est = rep(NA,K)
deviance.est = rep(NA,K)
gof.est = matrix(nrow=3, ncol=K)
dimnames(gof.est)[[1]] = c('gof','gof 2.5%','gof 97.5%')
recipCondNum = rep(NA,K)
zcrit = qnorm(p=.975)
for (k in 1:K) {   
    xcol = xmatCol[1:nx[k], k]
    X = matrix(xmat[,xcol], ncol=nx[k])
    beta.est[1:nx[k], k] = glm(ysum[,k]~X-1, family=poisson)$coefficients

    wcol = wmatCol[1:nw[k], k]
    W = matrix(wmat[,wcol], ncol=nw[k])
    alpha.est[1:nw[k], k] = 0

    paramGuess = c(as.vector(beta.est[1:nx[k],k]), as.vector(alpha.est[1:nw[k],k]))
    fit = optim(par=paramGuess, fn=negLL.MultinomPoissonMixture, method='BFGS', hessian=TRUE, yArg=y[,k,], xArg=X, wArg=W, Jarg=J)

    aic.est[k] = 2*fit$value + 2*length(fit$par)
    bic.est[k] = 2*fit$value + log(I)*length(fit$par)
    eigenVal = eigen(fit$hessian, only.values=TRUE)$values
    recipCondNum[k] = eigenVal[length(fit$par)]/eigenVal[1]

    beta.est[1:nx[k], k] = fit$par[1:nx[k]]
    alpha.est[1:nw[k], k] = fit$par[(nx[k]+1):(nx[k]+nw[k])]
    vcv = chol2inv(chol(fit$hessian))
    beta.se = sqrt(diag(vcv)[1:nx[k]])
    beta.lowerCL[1:nx[k], k] = fit$par[1:nx[k]] - zcrit*beta.se
    beta.upperCL[1:nx[k], k] = fit$par[1:nx[k]] + zcrit*beta.se
    alpha.se = sqrt(diag(vcv)[(nx[k]+1):(nx[k]+nw[k])])
    alpha.lowerCL[1:nw[k], k] = fit$par[(nx[k]+1):(nx[k]+nw[k])] - zcrit*alpha.se
    alpha.upperCL[1:nw[k], k] = fit$par[(nx[k]+1):(nx[k]+nw[k])] + zcrit*alpha.se
    
    deviance.est[k] = deviance.MultinomPoissonMixture(fit$par, ysumArg=ysum[,k], xArg=X, wArg=W, Jarg=J)
    gof.est[,k] = gof.MultinomPoissonMixture(deviance.est[k], 500, fit$par, xArg=X, wArg=W, Jarg=J)
    cat('Species ', species.Number[species.ind][k], ':  bic = ', bic.est[k], '  deviance = ', deviance.est[k], '  GOF = ', gof.est[,k], CR)
}




# create vector of species numbers used in analysis
speciesNumber = species.Number[species.ind]




# Compute MLE and quantiles for beta's sampling distribution

beta.ind = !is.na(beta.est)

postvec = beta.est[beta.ind]
beta.mle = matrix(nrow=length(nx), ncol=dim(xmat)[2])
dimnames(beta.mle) = list(speciesNumber, dimnames(xmat)[[2]])
cumsumnx = cumsum(nx)
for (i in 1:length(nx)) {
    xcol = xmatCol[1:nx[i], i]
    if (i>1) {
        beta.mle[i,xcol] = postvec[(cumsumnx[i-1]+1):cumsumnx[i]]
    }
    else {
        beta.mle[i,xcol] = postvec[1:nx[i]]
    }
}

postvec = beta.lowerCL[beta.ind]
beta.lower = beta.mle
for (i in 1:length(nx)) {
    xcol = xmatCol[1:nx[i], i]
    if (i>1) {
        beta.lower[i,xcol] = postvec[(cumsumnx[i-1]+1):cumsumnx[i]]
    }
    else {
        beta.lower[i,xcol] = postvec[1:nx[i]]
    }
}
postvec = beta.upperCL[beta.ind]
beta.upper = beta.mle
for (i in 1:length(nx)) {
    xcol = xmatCol[1:nx[i], i]
    if (i>1) {
        beta.upper[i,xcol] = postvec[(cumsumnx[i-1]+1):cumsumnx[i]]
    }
    else {
        beta.upper[i,xcol] = postvec[1:nx[i]]
    }
}

beta.nonzeroInd = (beta.lower<0 & beta.upper<0) | (beta.lower>0 & beta.upper>0)



# Compute MLE and quantiles for alpha's sampling distribution

alpha.ind = !is.na(alpha.est)


postvec = alpha.est[alpha.ind]
alpha.mle = matrix(nrow=length(nw), ncol=dim(wmat)[2])
dimnames(alpha.mle) = list(speciesNumber, dimnames(wmat)[[2]])
cumsumnw = cumsum(nw)
for (i in 1:length(nw)) {
    wcol = wmatCol[1:nw[i], i]
    if (i>1) {
        alpha.mle[i,wcol] = postvec[(cumsumnw[i-1]+1):cumsumnw[i]]
    }
    else {
        alpha.mle[i,wcol] = postvec[1:nw[i]]
    }
}

postvec = alpha.lowerCL[alpha.ind]
alpha.lower = alpha.mle
for (i in 1:length(nw)) {
    wcol = wmatCol[1:nw[i], i]
    if (i>1) {
        alpha.lower[i,wcol] = postvec[(cumsumnw[i-1]+1):cumsumnw[i]]
    }
    else {
        alpha.lower[i,wcol] = postvec[1:nw[i]]
    }
}
postvec = alpha.upperCL[alpha.ind]
alpha.upper = alpha.mle
for (i in 1:length(nw)) {
    wcol = wmatCol[1:nw[i], i]
    if (i>1) {
        alpha.upper[i,wcol] = postvec[(cumsumnw[i-1]+1):cumsumnw[i]]
    }
    else {
        alpha.upper[i,wcol] = postvec[1:nw[i]]
    }
}

alpha.nonzeroInd = (alpha.lower<0 & alpha.upper<0) | (alpha.lower>0 & alpha.upper>0)




# Tabulate MLEs  using Askins' ordering of species
d.species = read.csv('Table_with_species_list.csv')
outInd = match(d.species[,'Species.No.'], speciesNumber)
commonName = as.character(d.species[,'Common.name'])
genusAndSpecies = as.character(d.species[,'Scientific.name'])

beta.out = round(beta.mle[outInd, -c(8,9)], 2)
write.table(cbind(commonName, beta.out), file='beta.tex', row.names=FALSE, col.names=TRUE, sep=' & ', na=' ', quote=FALSE, eol=' \\ \n')


alpha.out = round(alpha.mle[outInd, c(1,4)], 2)
write.table(cbind(commonName, alpha.out), file='alpha.tex', row.names=FALSE, col.names=TRUE, sep=' & ', na=' ', quote=FALSE, eol=' \\ \n')
